<?
/*
*******************************
 IP-SYMCON Event Scripting
*******************************
File     : Temperatur_anzeigen.ips.php
Trigger  : Temperatur
Interval : OnChange
Author   : HJH
Project  : Thermometer
Date     : 11.11.07
*/



//******************************************************************************
//
// Dieses Script bringt einen Temperaturwert auf dem Bild eines
// "Messinstruments" im Designer zur Anzeige.
//
// Bei jedem Aufruf wird aus den Bildern "TempMeter.png" und "TempPointer.png"
// ein neues Bild "Thermometer.png" erzeugt.
//
// Voraussetzungen:
// 1 - Die Variable "Temperatur" ist mit einem Temperatursensor verknpft.
// 2 - Das Bild "Thermometer.png" ist in den ImagePool des Designers
//     aufgenommen worden.
//
// Unter "Konfiguration" knnen noch einige Eigenschaften angepasst werden.
//
//******************************************************************************

// Konfiguration ---------------------------------------------------------------
// Temperaturwert als Zeiger oder als Balken anzeigen
$temPtr = true;
// Extremwerte als schattierte Flchen anzeigen
$extBar = true;
// Extremwerte als Zeiger anzeigen
$extPtr = false;
// zustzlichen Text einblenden
$text = "Balkon";
// -----------------------------------------------------------------------------



// Pfade und Dateien
$path = IPS_GetKernelDir(). "scripts\\Thermometer\\";
$imgPath = $path. "Images\\";
//$imgCachePath = IPS_GetKernelDir(). "imgcache\\";
$meterFile = $imgPath. "TempMeter.png";      // Bild des Anzeigeinstruments
$pointerFile = $imgPath. "TempPointer.png";  // Bild des Zeigers
$extPtrFile = $imgPath. "TempExtPointer.png";// Bild des Min/Max-Zeigers
$poolFile = $path. "Thermometer.png";        // neu erzeugtes Bild fr den Designer

// Skalendaten
$sl = 300;  // Skalenlnge in Pixeln
$t1 = 0;    // untere Anzeigegrenze in C
$t2 = 30;   // obere Anzeigegrenze in C

// Farben
$transparent = 0x123456;   // transparente Farbe
$fcolor = 0xffffff;        // Schriftfarbe (wei)
$grey = 0xc4c4c4;          // Textfarbe (grau)



// aktuelle Temperaturwerte holen
$tcur = GetValueFloat("Temperatur");

// Reset-Buttons bedienen
if ($IPS_SENDER == "Designer")
{
 switch ($IPS_COMPONENT)
 {
  case "Reset_Min":
   // Wert auf aktuelle Temperatur setzen
   SetValueFloat("Temperatur_min", $tcur);
   break;
  case "Reset_Max":
   // Wert auf aktuelle Temperatur setzen
   SetValueFloat("Temperatur_max", $tcur);
   break;
 }
}

$tmin = GetValueFloat("Temperatur_min");
$tmax = GetValueFloat("Temperatur_max");

// Temperaturwerte an Skala anpassen
$tval = WertSkalieren($tcur, $t1, $t2, $sl);
$tvmn = WertSkalieren($tmin, $t1, $t2, $sl);
$tvmx = WertSkalieren($tmax, $t1, $t2, $sl);

// Zeigerausschlag begrenzen
$tval = ($tval < 0) ? 0 : (($tval > $sl) ? $sl : $tval);
$tvmn = ($tvmn < 0) ? 0 : (($tvmn > $sl) ? $sl : $tvmn);
$tvmx = ($tvmx < 0) ? 0 : (($tvmx > $sl) ? $sl : $tvmx);

// Bild des Anzeigeinstruments laden
$img = imagecreatefrompng($meterFile);

// Temperatur als Balken anzeigen
if (!$temPtr)
{
 $img9 = imagecreatetruecolor($sl, 1);
 $ofs = 10;
 imagecopymerge($img, $img9, $ofs+66+$tval, 73, 0, 0, $sl+$ofs-$tval, 30, 60);
 imagedestroy($img9);
}

// schattierte Balken fr Extremwertanzeige
if ($extBar)
{
 $img9 = imagecreatetruecolor($sl, 1);
 $ofs = 10;
 imagecopymerge($img, $img9, 65, 73, 0, 0, $ofs+$tvmn, 30, 15);
 imagecopymerge($img, $img9, $ofs+66+$tvmx, 73, 0, 0, $sl+$ofs-$tvmx, 30, 15);
 imagedestroy($img9);
}

// Extremwert-Zeiger positionieren
if ($extPtr)
{
 ZeigerSetzen($img, 65+$tvmn+1, 73, $extPtrFile, $transparent);   // Min-Zeiger
 ZeigerSetzen($img, 65+$tvmx+1, 73, $extPtrFile, $transparent);   // Max-Zeiger
}

// Temperatur als Zeiger anzeigen
if ($temPtr)
 ZeigerSetzen($img, 65+$tval+1, 73, $pointerFile, $transparent);   // Temp-Zeiger

// Werte einblenden
WertSetzen($img, 224, 31, $tcur, "arialbd.ttf", 15.0, $fcolor);
WertSetzen($img,  32, 94, $tmin,   "arial.ttf", 11.0, $fcolor);
WertSetzen($img, 417, 94, $tmax,   "arial.ttf", 11.0, $fcolor);

// Text einblenden
$szText = imageftbbox(13.0, 0.0, "arialbd.ttf", $text);
imagefttext($img, 13.0, 0.0, imagesx($img)-14-$szText[2], 28, $grey, "arialbd.ttf", $text);

// das gerade erzeugte Bild fr den Designer abspeichern
imagepng($img, $poolFile);

// den vom Bild belegten Speicher wieder freigeben
imagedestroy($img);

// Designer informieren, nderung anzeigen
IMGP_SendEvent("Thermometer.png");



// Funktionen ------------------------------------------------------------------

//******************************************************************************
// Zeiger einkopieren
// ==================
//
// Parameter:
// $img        : Bild in das hineinkopiert werden soll
// $px, $py    : Koordinaten, an die kopiert werden soll
// $obj        : PNG-Dtei (inkl. Pfad) des zu kopierenden Objekts (Zeigerbild)
// $trans      : Farbe, die als transparent gelten soll
//
//------------------------------------------------------------------------------
function ZeigerSetzen($img, $px, $py, $obj, $trans)
{
 $ptr = imagecreatefrompng($obj);      // Bild des Zeigers anlegen
 $ptrw = imagesx($ptr);                // Breite des Bildes
 $ptrh = imagesy($ptr);                // Hhe des Bildes
 imagecolortransparent($ptr, $trans);  // transparente Farbe festlegen
 imagecopymerge($img, $ptr, $px, $py, 0, 0, $ptrw, $ptrh, 100);
 imagedestroy($ptr);                   // Bildspeicher wieder freigeben
}
//******************************************************************************



//******************************************************************************
// Wert einkopieren
// ================
//
// Parameter:
// $img        : Bild in das hineinkopiert werden soll
// $px, $py    : Koordinatenzentrum
// $val        : Wert, der eingeblendet werden soll
// $font       : Schriftart
// $size       : Schriftgre
//
//------------------------------------------------------------------------------
function WertSetzen($img, $px, $py, $val, $font, $size, $color)
{
 $vals = sprintf("%4.1f", $val);
 $sz = imageftbbox($size, 0.0, $font, $vals);
 $w2 = $sz[2]>>1;
 imagefttext( $img, $size, 0.0, $px-$w2, $py, $color, $font, $vals);
}
//******************************************************************************



//******************************************************************************
// Wert skalieren
// ==============
//
// Parameter:
// $val        : Wert, der skaliert werden soll
// $sb, $se    : Anfangs- und Endwert der Skala
// $sl         : Skalenlnge in Pixeln
//
//------------------------------------------------------------------------------
function WertSkalieren($val, $sb, $se, $sl)
{
 return (integer)round(($val-$sb) * ($sl/($se-$sb)));
}
//******************************************************************************
?>
